"use strict";

let crypto = require('crypto'),
    ByteBuffer = require('bytebuffer');

let ZetaSignatureGenerator = function(publicKey, privateKey, zetaPublicKey){
    this.publicKey = publicKey;
    this.privateKey = privateKey;
    this.zetaPublicKey = zetaPublicKey;
};

function generate(inputMap) {
    let input = flattenObject(inputMap, '');
    let sign = crypto.createSign('SHA256');
    sign.update((new Buffer(input)).toString('utf8'));
    let bufferedSignature = sign.sign(this.privateKey);
    let shaSum = crypto.createHash('sha256');
    shaSum.update(this.publicKey);
    let publicKeyDigest = shaSum.digest();
    let PKI = toByteArray(publicKeyDigest).slice(0, 3);
    let l = toByteArray(bufferedSignature).byteLength;
    let zetaSignature = new ByteBuffer(2 + 3 + l);
    let IDB = 1;
    zetaSignature.writeShort(IDB);
    zetaSignature.append(PKI);
    zetaSignature.append(toByteArray(bufferedSignature));
    return (zetaSignature.buffer).toString('base64');
}

function flattenObject(inputMap,prefix) {
    if(prefix !== ''){prefix = `${prefix}.`}
    let sortedKeys = Object.keys(inputMap).sort();
    let stringArray = sortedKeys.map(function (key) {
        if(inputMap[key] instanceof Object){
            return flattenObject(inputMap[key], `${prefix}${key}`);
        }
        else if(inputMap[key] instanceof Array){
            inputMap[key].forEach(function(k,i){
                return flattenObject(inputMap[key][k], `${prefix}${key}${i}.`);
            });

        }
        else {
            return (`${prefix}${key}=${inputMap[key]}`);
        }
    });
    return stringArray.join(`\n`);
}

function toByteArray(buffer) {
    let ab = new ArrayBuffer(buffer.length);
    let view = new Uint8Array(ab);
    for (let i = 0; i < buffer.length; ++i) {
        view[i] = buffer[i];
    }
    let myString = "";
    for (let i=0; i<view.length; i++) {
        myString += String.fromCharCode(view[i])
    }
    return ab;
}

ZetaSignatureGenerator.prototype.generate = generate;
module.exports = ZetaSignatureGenerator;
