"use strict";

let https = require('https'),
    config = require('./config');

function mergeObject(objectToMergeInto, objectToMergeFrom) {
    if (!objectToMergeFrom || Object.keys(objectToMergeFrom).length === 0) {
        return objectToMergeInto;
    }
    Object.keys(objectToMergeFrom).forEach(function (key) {
        objectToMergeInto[key] = objectToMergeFrom[key];
    });
    return objectToMergeInto;
}

module.exports = {

    makeAPICall: function (request, path) {
        let headers = {
            'Content-Type': 'application/json',
            'User-Agent': 'PwZ Sample',
            "X-Zeta-Authtoken": config.authToken
        };
        let httpParams = {
            host: config.PWZ_HOST,
            path: path,
            method: "POST",
            headers: headers
        };
        console.log(httpParams, request);
        return new Promise(function (resolve, reject) {
            try {
                let post_req = https.request(httpParams, function (res) {
                    res.setEncoding('utf8');
                    let responseStatus = parseInt(res.statusCode);
                    let response = '';
                    res.on('data', function (chunk) {
                        response += chunk;
                    });
                    res.on('end', function () {
                        let responseHeaders = res.headers;
                        let output = response;
                        if(responseHeaders["content-type"] && responseHeaders["content-type"].includes("application/json")){
                            output = JSON.parse(response);
                        }
                        if (responseStatus !== 200) {
                            reject(output);
                        }
                        return resolve(output);
                    });
                    res.on('error', function (e) {
                        return reject(e);
                    });
                });
                post_req.write(JSON.stringify(request));
                post_req.end();
            }
            catch (error) {
                return reject(error);
            }
        });
    },
    getZetaMobileNumberUrl: function (mobileNumber) {
        if (mobileNumber.length === 10) {
            return "p:+91" + mobileNumber;
        }
        if (mobileNumber.startsWith("+91") && mobileNumber.length() == 13) {
            return "p:" + mobileNumber;
        }
        if (mobileNumber.startsWith("p:+91") && mobileNumber.length() == 15) {
            return mobileNumber;
        }
        throw new Error("Invalid mobile number format. Correct format +9199XXXXXXXXXX or 99XXXXXXXXXX");
    }
};